# -*- coding: utf-8 -*- 
# @Time : 2020-06-08 09:28 
# @Author : zhengjinlei 
# @File : report_utils.py
import json
import logging
from datetime import datetime

from django.db.models import Sum

from inspect_report.models import RulesStat, CheckSession, Team, Seat, SeatStat, ScoreStat, ScoreTemplate, ScoreItems
from config.config import TABLE_PRE

logger = logging.getLogger('app_file')


def get_inspect_rule(task):
    """
    获取质检规则
    :param task:
    :return:
    """
    service_score, business_score, validate_zero = 0, 0, 0
    score_item_service, score_item_business, score_item_zero = {}, {}, {}
    extra = json.loads(task['extra']) if task['extra'] else {}
    template_id = extra['templateId']
    score_item_ids = extra['scoreItemIds']
    try:
        template = ScoreTemplate.objects.get(pk=template_id)
    except Exception as e:
        logger.error('obtain score template fail!, error: %s', e)
        return 0, 0, 0, {}, {}, {}
    rules = json.loads(template.rules)
    for rule in rules:
        if rule['type'] == '服务类':
            service_score = ScoreItems.objects.filter(id__in=rule['scoreItemIds']).aggregate(ss=Sum('score'))['ss']
            service_items = list(set(rule['scoreItemIds']).intersection(set(score_item_ids)))
            items = ScoreItems.objects.filter(id__in=service_items).values('name', 'score', 'scoreType')
            for item in items:
                score_item_service[item['name']] = item['score'] if item['scoreType'] == 1 else -item['score']
        elif rule['type'] == '业务类':
            business_score = ScoreItems.objects.filter(id__in=rule['scoreItemIds']).aggregate(ss=Sum('score'))['ss']
            business_items = list(set(rule['scoreItemIds']).intersection(set(score_item_ids)))
            items = ScoreItems.objects.filter(id__in=business_items).values('name', 'score', 'scoreType')
            for item in items:
                score_item_business[item['name']] = item['score'] if item['scoreType'] == 1 else -item['score']
        elif rule['type'] == '0分项':
            zero_items = list(set(rule['scoreItemIds']).intersection(set(score_item_ids)))
            items = ScoreItems.objects.filter(id__in=zero_items).values('name', 'score', 'scoreType')
            for item in items:
                score_item_zero[item['name']] = item['score'] if item['scoreType'] == 1 else -item['score']
    return service_score, business_score, validate_zero, score_item_service, score_item_business, score_item_zero


def get_remain(check):
    remain = 0
    if check['startTime'] and check['closeTime']:
        start = datetime.strptime(check['startTime'], '%Y-%m-%d %H:%M:%S').timestamp()
        close = datetime.strptime(check['closeTime'], '%Y-%m-%d %H:%M:%S').timestamp()
        remain = int(close - start)
    return remain


def team_seat():
    teams = Team.objects.values_list('id', 'name')
    team_id_name = {team[0]: team[1] for team in teams}
    seats = Seat.objects.values_list('code', 'team_id')
    seat_code_team_name = {seat[0]: team_id_name[seat[1]] for seat in seats}
    return seat_code_team_name


def single_rules_stat(t, create_date, stat_count=0):
    if not RulesStat.objects.filter(taskId=t['id']).exists():
        table_name = TABLE_PRE + t['sessionCollectionId']
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId': t['id'], 'violationRuleCount__gt': 0}
        checks = tn.objects.filter(**session_condition).values('id', 'checkResult', 'agentName', 'customName', 'score',
                                                               'remainTime', 'startTime', 'closeTime', 'taskId',
                                                               'sessionId')
        for check in checks:
            result = check['checkResult']
            if result:
                rules_list = []
                data = json.loads(result)
                for d in data:
                    remain = get_remain(check)
                    if 'isViolation' in d.keys() and d['isViolation'] and 'rule' in d.keys() and 'name' in d['rule'].keys():
                        name = d['rule']['name']

                        rule_obj = {'create_date': create_date, 'sessionCollectionId': t['sessionCollectionId'],
                                    'rule': name, 'rule_num': 1,
                                    'task': team_seat().get(check['agentName'], '未找到团队'),
                                    'agentName': check['agentName'], 'customName': check['customName'],
                                    'score': check['score'], 'remainTime': remain,
                                    'taskId': check['taskId'], 'sessionId': check['sessionId'],
                                    'session_table_id': check['id']}
                        rules_list.append(RulesStat(**rule_obj))
                        stat_count += 1
                logger.info('rule stat num: [%s]', stat_count)
                RulesStat.objects.bulk_create(rules_list)


def single_seat_stat(t, create_date, stat_count=0):
    if not SeatStat.objects.filter(taskId=t['id']).exists():
        table_name = TABLE_PRE + t['sessionCollectionId']
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId': t['id']}
        checks = tn.objects.filter(**session_condition).values('id', 'agentName', 'remainTime', 'startTime',
                                                               'closeTime', 'taskId', 'violationRuleCount')
        seat_dict = {}
        seat_list = []
        for check in checks:
            stat_count += 1
            remain = get_remain(check)
            if check['agentName'] in seat_dict.keys():
                seat = seat_dict.get(check['agentName'])
                seat['total_session'] += 1
                seat['remainTime'] += remain
                seat['validate_session'] += 1 if check['violationRuleCount'] > 0 else 0
            else:
                seat_dict[check['agentName']] = {'create_date': create_date, 'agentName': check['agentName'],
                                                 'taskId': check['taskId'],
                                                 'task': team_seat().get(check['agentName'], '未找到团队'),
                                                 'sessionCollectionId': t['sessionCollectionId'], 'total_session': 1}
                seat_dict[check['agentName']]['validate_session'] = 1 if check['violationRuleCount'] > 0 else 0
                seat_dict[check['agentName']]['remainTime'] = remain
        for k, v in seat_dict.items():
            seat_obj = {'agentName': k}
            seat_obj.update(v)
            seat_list.append(SeatStat(**seat_obj))
        SeatStat.objects.bulk_create(seat_list)


def single_score_stat(t, create_date, stat_count=0):
    if not ScoreStat.objects.filter(taskId=t['id']).exists():
        table_name = TABLE_PRE + t['sessionCollectionId']
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId': t['id']}
        checks = tn.objects.filter(**session_condition).values('id', 'agentName', 'taskId', 'violationRuleCount',
                                                               'startTime', 'score', 'scoreItemRecord')
        seat_dict = {}
        seat_list = []
        service_s, business_s, vz, score_item_service, score_item_business, score_item_zero = get_inspect_rule(t)
        for check in checks:
            stat_count += 1
            score_item = json.loads(check['scoreItemRecord'])
            service_score = service_s
            business_score = business_s
            validate_zero = 0

            for m in score_item:
                if m['scoreItemName'] in score_item_service.keys():
                    service_score -= m['score']
                if m['scoreItemName'] in score_item_business.keys():
                    business_score -= m['score']
                if m['scoreItemName'] in score_item_zero.keys():
                    validate_zero += 1
            if validate_zero > 0:
                score = 0
            else:
                score = service_score + business_score
            if check['agentName'] in seat_dict.keys():
                seat = seat_dict.get(check['agentName'])
                seat['total_session'] += 1
                seat['validate_session'] += 1 if check['violationRuleCount'] > 0 else 0
                seat['validate_num'] += len(score_item)
                seat['score'] = round((seat['score'] + score) / 2, 2)
                seat['service_score'] = round((seat['service_score'] + service_score) / 2, 2)
                seat['business_score'] = round((seat['business_score'] + business_score) / 2, 2)
            else:
                seat_dict[check['agentName']] = {'create_date': create_date, 'agentName': check['agentName'],
                                                 'taskId': check['taskId'],
                                                 'task': team_seat().get(check['agentName'], '未找到团队'),
                                                 'sessionCollectionId': t['sessionCollectionId'], 'total_session': 1,
                                                 'score': score, 'service_score': service_score,
                                                 'business_score': business_score, 'validate_zero': validate_zero}
                seat_dict[check['agentName']]['validate_session'] = 1 if check['violationRuleCount'] > 0 else 0
                seat_dict[check['agentName']]['validate_num'] = len(score_item)
        for k, v in seat_dict.items():
            seat_obj = {'agentName': k}
            seat_obj.update(v)
            seat_list.append(ScoreStat(**seat_obj))
        ScoreStat.objects.bulk_create(seat_list)


def re_stat_by_task(task):
    single_rules_stat(task, task['createdAt'], 0)
    single_seat_stat(task, task['createdAt'], 0)
    single_score_stat(task, task['createdAt'], 0)


def delete_stat_by_task(task):
    RulesStat.objects.filter(taskId=task.id).delete()
    SeatStat.objects.filter(taskId=task.id).delete()
    ScoreStat.objects.filter(taskId=task.id).delete()
